<?php

namespace Bydedi\Payment;

use com\realexpayments\remote\sdk\domain\Card;
use com\realexpayments\remote\sdk\domain\CardType;
use com\realexpayments\remote\sdk\domain\PresenceIndicator;
use com\realexpayments\remote\sdk\domain\payment\AutoSettle;
use com\realexpayments\remote\sdk\domain\payment\AutoSettleFlag;
use com\realexpayments\remote\sdk\domain\payment\PaymentRequest;
use com\realexpayments\remote\sdk\domain\payment\PaymentResponse;
use com\realexpayments\remote\sdk\domain\payment\PaymentType;
use com\realexpayments\remote\sdk\http\HttpConfiguration;
use com\realexpayments\remote\sdk\RealexClient;
use Tpe;
use Commande;
use Panier;

/**
 * Class RealexDirect
 * @package Bydedi\Payment
 */
class RealexDirect
{
    const ENDPOINT_TEST = "https://test.realexpayments.com/epage-remote.cgi";

    const ENDPOINT_SANDBOX = "https://api.sandbox.realexpayments.com/epage-remote.cgi";

    /**
     * @var string
     */
    protected $mode;

    /**
     * @var string
     */
    protected $merchantId;

    /**
     * @var string
     */
    protected $secret;

    /**
     * @var string
     */
    protected $account;

    /**
     * @var HttpConfiguration
     */
    protected $httpConfiguration = null;

    /**
     * @var string
     */
    protected $currency;

    /**
     * @var CreditCardTypeTranslator|null
     */
    protected $creditCardTypeTranslator;

    /**
     * RealexDirect constructor.
     * @param $config
     * @param null $creditCardTypeTranslator
     */
    public function __construct($config, $creditCardTypeTranslator = null)
    {
        // sandbox avec un compte chez Realex
        if ($config['realex']['mode'] != 'prod') {
            $this->httpConfiguration = new HttpConfiguration();
            $this->httpConfiguration->setEndpoint(self::ENDPOINT_SANDBOX);
        }

        // mode test : si pas de compte créer chez Realex
        if ($config['realex']['mode'] == 'test') {
            $config['realex']['merchant_id'] = "realexsandbox";
            $config['realex']['account'] = 'internet';
            $config['realex']['secret'] = 'Po8lRRT67a';

            $this->httpConfiguration = new HttpConfiguration();
            $this->httpConfiguration->setEndpoint(self::ENDPOINT_TEST);
        }

        $this->merchantId = $config['realex']['merchant_id'];
        $this->secret = $config['realex']['secret'];
        $this->account = $config['realex']['account'];

        $this->currency = 'GBP';

        $this->creditCardTypeTranslator =
            (is_null($creditCardTypeTranslator)) ? new CreditCardTypeTranslator() : $creditCardTypeTranslator;
    }

    /**
     * @param $currency
     * @return $this
     */
    public function setCurrency($currency)
    {
        $this->currency = $currency;
        return $this;
    }

    /**
     * @param Tpe $tpe
     * @param $amount
     * @param $cardNumber
     * @param $cardExpirationDate
     * @param $cardCrypto
     * @param $cardName
     *
     * @return mixed
     */
    public function doAuthorization(Tpe $tpe, $amount, $cardNumber, $cardExpirationDate, $cardCrypto, $cardName)
    {
        // TODO: Implement doAuthorization() method.
    }

    /**
     * @param Tpe $tpe
     * @param $amount
     * @param $cardNumber
     * @param $cardExpirationDate
     * @param $cardCrypto
     * @param $cardName
     *
     * @return mixed
     */
    public function doDebit(Tpe $tpe, $amount, $cardNumber, $cardExpirationDate, $cardCrypto, $cardName)
    {

        $type = $this->creditCardTypeTranslator->getFromCardNumber($cardNumber);

        $amount = $amount * 100;
        $amount = round($amount, 2);

        $card = ( new Card() )
            ->addType($type)
            ->addNumber($cardNumber)
            ->addExpiryDate($cardExpirationDate)
            ->addCvn($cardCrypto)
            ->addCvnPresenceIndicator(PresenceIndicator::CVN_PRESENT)
            ->addCardHolderName($cardName);

        $request = (new PaymentRequest())
            ->addMerchantId($this->merchantId)
            ->addAccount($this->account)
            ->addType(PaymentType::AUTH)
            ->addCard($card)
            ->addAmount($amount)
            ->addCurrency($this->currency)
            ->addAutoSettle((new AutoSettle() )->addFlag(AutoSettleFlag::TRUE));

        $client = new RealexClient($this->secret, $this->httpConfiguration);
        $response['bank_response'] = $client->send($request);
        $response['order_id'] = $tpe->tpe_reference_id;

        return $this->response($response);

    }

    /**
     * @param array $request
     *
     * @return mixed
     */
    public function response($request)
    {
        $return = ['status' => false];

        if ($request['bank_response']->isHashValid($this->secret) == false) {
            $return['status'] = false;
            $return['tpe_reference_id'] = $request['order_id'];
            $return['message'] = 'Hash invalid';
            return $return;
        }

        if (!empty($request['order_id'])) {
            $tpe = new Tpe;

            // cas response ok
            if ($request['bank_response']->isSuccess()) {

                $commande = new Commande();
                try {
                    $return['commande_id'] = $commande->saveFromPanier(Panier::getInstance());
                    $tpe->assoc($request['order_id'], $return['commande_id']);
                } catch (\Exception $e) {
                    $return['status'] = false;
                    $return['tpe_reference_id'] = $request['order_id'];
                    return $return;
                }

                $return['status'] = true;
                $return['tpe_reference_id'] = $request['order_id'];
            } else {
                $return = array_merge($return, $request);
            }

            // on prepare la réponse pour sauvegarde en base
            $request['bank_response'] = (array) $request['bank_response'];

            $expected = ['timeStamp', 'merchantId', 'account', 'orderId', 'result', 'authCode', 'message',
                'paymentsReference', 'cvnResult', 'batchId', 'hash', 'avsPostcodeResponse', 'avsAddressResponse'];

            foreach ($request['bank_response'] as $k => $v) {
                $newKey = str_replace('com\realexpayments\remote\sdk\domain\payment\PaymentResponse', '', trim($k));
                $newKey = trim($newKey);

                if (in_array($newKey, $expected)) {
                    $request['bank_response'][$newKey] = $v;
                }

                unset($request['bank_response'][$k]);
            }

            $request = array_merge($request, $request['bank_response']);

            unset($request['bank_response']);

            $tpe->addResponse($request['order_id'], $request);
        }

        return $return;
    }

    /**
     * @param $params
     * @return mixed
     */
    public function sendRequest($params)
    {
        // TODO: Implement sendRequest() method.
    }
}
